#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sound/sound.h>
#include <sound/sounddriver.h>

/* Audio stream player for mulaw 8KHz sounds for NeXTSTEP/OpenStep
   Written 1997 by Frank Siegert, frank@this.net, http://www.this.net/~frank

   May be used for any purpose as long as this header is not removed from
   the source code.  
   
   This whole thing is a trick, since the braindead NeXT Intel hardware 
   cannot play sounds smaller than vm_page_size reliable we first read 
   the input as mulaw, 8 Khz - then convert it to 22.5 kHz Stereo, then 
   playing it. This resolves most troubles as the sound hardware and the 
   driver gets what it needs. 
*/

/* Attention Do NOT change this constant, it is hand tuned, since 2976 
   mulaw, 8kHz samples will give 8192 16 bit stereo 22050 kHz samples 
   after conversion. 
*/

#define PLAYBUFSIZE 2976

int queued=0;

int snd_end(SNDSoundStruct *s, int tag, int err)
{
    SNDFree(s);
    queued--;
}

int main () {
	static int playbufptr=0,playtag=2000,len,curFR=0;
  	static unsigned char *work, *buf;
  	static SNDSoundStruct *snd0buf,*sndbuf[10];
	static unsigned char data[128];
	SNDReserve(SND_ACCESS_OUT,15);
	SNDAlloc(&snd0buf, PLAYBUFSIZE,SND_FORMAT_MULAW_8, SND_RATE_CODEC,1,4);
	while (!feof(stdin)) {
		len= fread(data,1,128,stdin);
		buf=data;
		while (len) {
			len--;
  			if (playbufptr==0) {
			  	work=(unsigned char*)snd0buf+snd0buf->dataLocation;
		  		playbufptr=PLAYBUFSIZE;
  			} 
			*work++ = *buf++;
			if (--playbufptr == 0) {
				while (queued>8) usleep(100000);
				SNDAlloc(&sndbuf[curFR], 0, SND_FORMAT_LINEAR_16, SND_RATE_LOW,1,4);
				SNDConvertSound(snd0buf,&sndbuf[curFR]);
  				SNDStartPlaying(sndbuf[curFR],++playtag,5,0,0,(SNDNotificationFun)snd_end);
				queued++;
				playbufptr=0;
				curFR++; if (curFR>9) curFR=0;
			}
  		}
	}
	while (playbufptr>0) {
			*work++ = 0;
			if (--playbufptr == 0) {
				while (queued>8) usleep(100000);
  				SNDStartPlaying(sndbuf[curFR],++playtag,5,0,0,(SNDNotificationFun)snd_end);
				break;
			}
	}
	while (queued>0) usleep(100000);
	SNDUnreserve(SND_ACCESS_OUT);
	return 0;
}
