#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sound/sound.h>
#include <sound/sounddriver.h>

/* Audio stream recorder for mulaw 8KHz sounds for NeXTSTEP/OpenStep
   Written 1997 by Frank Siegert, frank@this.net, http://www.this.net/~frank

   May be used for any purpose as long as this header is not removed from
   the source code.  
   
 */

static int playtag=3,queued=0;

mutex_t sndMutex;
condition_t sndCondition;

SNDNotificationFun start_record(SNDSoundStruct *s, int tag, int err)
{
	/* do nothing at the moment */
}

SNDNotificationFun end_record(SNDSoundStruct *s, int tag, int err)
{
	int i;
	unsigned char *work;
    	condition_broadcast(sndCondition);
	work=(unsigned char*)s+s->dataLocation;
	i=s->dataSize;
	while (i--) {
		putc(*work++,stdout);
	}
	queued--;
	return (SNDFree(s));
}

/* This is a trick, vm_page_size/2 is the lowest snd buffer size 
   where recording can happen without click (in case you have troubles
   resize this to vm_page_size - it may resove your problems
*/

#define RECBUFSIZE vm_page_size/2

int main ()
{
    int curFR=0;
    SNDSoundStruct *sndbuf[3];

    sndCondition=condition_alloc();
    sndMutex=mutex_alloc();
    while (1) {
	int i,err, timecall;

	SNDAlloc(&sndbuf[curFR], RECBUFSIZE, SND_FORMAT_MULAW_8, SND_RATE_CODEC,1,4);

	err = SNDStartRecording(sndbuf[curFR], ++playtag ,255, 0, (SNDNotificationFun)start_record, (SNDNotificationFun)end_record);	
	if (err) {
		/* if we die here due to strange errors we are immediately resurrected by
  		   sfmikes process management, we only loose a frame or less */
		exit(-1);
	}
	queued++;
	curFR++; if (curFR>2) curFR=0;
	if (queued>1) {
		mutex_lock(sndMutex);	
		/* waiting for finish - SNDWait is potential evil on Intel hardware, we do it ourself */
		condition_wait(sndCondition, sndMutex);
		mutex_unlock(sndMutex);
	}
   } 
   return 0; /* never */
}
