#include "speakfree.h"

/* reworked sound routines for NS Intel hardware by Frank Siegert [frank@this.net] */
/* partly based on the work by Roberto Di Cosmo, Paris, France */

#ifdef NeXT

#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

int childPid;
int fpToChild;
int fromChild;
int EternalLife;

static int Recording=0;

#define RINGBUFSIZE 120000
static int InsertRPos;
static int GetRPos;
unsigned char ringbuffer[RINGBUFSIZE+1];

/*  SOUNDINIT  --  Open the sound peripheral and initialise for
		   access.  Return TRUE if successful, FALSE
		   otherwise.  */

int soundinit(int iomode)
{
	if (iomode==O_WRONLY) { /* speaker or mike */
		startIOSubprocess("sndsplay");
	} else {
		startIOSubprocess("sndsrec");
	}
 	return TRUE;
}

int dataFromChild(int sig)
{
	unsigned char buffer[1024];
	int len, total=0;
	//printf("Child sent data %d\n",sig);
	while ((len=read(fromChild, buffer, 1023))>0) {
		int i=len;
		unsigned char *work=buffer;
		//printf("Got Len %d\n",len);
		while (i--) {
			ringbuffer[InsertRPos++]=*work++;
			if (InsertRPos>=RINGBUFSIZE) InsertRPos=0;
			total++;
		}
	}
	if (total<1) {
		//printf("Child DIED\n");
		if (EternalLife) startIOSubprocess(NULL);
	}
	return 0;
}

void sendDataToChild(unsigned char *data, int len)
{
	int ret;
	if ((ret= fwrite(data,1, len, fpToChild))!=len) {
		fprintf(stderr,"Write to PID %d failed (%d)\n", childPid, ret);
	}
}

int startIOSubprocess(char *name)
{
    static char acProcess[1024];
    int pipeTo[2];
    int pipeFrom[2];
    int	tty, numFds, fd;
    int processGroup;
    int pidChild;

   if (name) {
	strcpy(acProcess,name);
	EternalLife=TRUE;
   } else
   	name=acProcess;

   if (pipe(pipeTo) < 0 || pipe(pipeFrom) < 0) {
	fprintf(stderr,"Error starting UNIX pipes to subprocess.\n");
	return (-1);
    }
    
    switch (pidChild = vfork())
    {
    case -1:	/* error */
	fprintf(stderr,"Error starting UNIX vfork of subprocess.\n");
	return (-1);

    case 0:	/* child */
	dup2(pipeTo[0], 0);
	dup2(pipeFrom[1], 1);
	/* dup2(pipeFrom[1], 2); */
	
	numFds = getdtablesize();
	for (fd=3; fd<numFds; fd++)
	    close(fd);

	processGroup = getpid();
	ioctl(0, TIOCSPGRP, (char *)&processGroup);
	setpgrp (0, processGroup);
	
	/* we exec a /bin/sh so that cmds are easier to specify for the user */
	execl(name, name, 0);
	perror("vfork (child)"); // should never gets here tho
	exit(1);

    default:	/* parent */
	
	signal(SIGIO, dataFromChild);

	childPid = pidChild;

	close(pipeTo[0]);
	close(pipeFrom[1]);
    
	fpToChild = fdopen(pipeTo[1], "w");
	fromChild = pipeFrom[0];

	fcntl(fromChild, F_SETOWN, getpid());
	fcntl(fromChild, F_SETFL, FASYNC|FNDELAY);

	setbuf(fpToChild, NULL);
	//printf("Child STARTED\n");
	}
	return 0;
}
	

/*  SOUNDTERM  --  Close the sound device.  */

void soundterm()
{
	EternalLife=FALSE;
 	//fprintf(stderr, "Killing %d\n", childPid);
	kill(childPid, 1);
 	//fprintf(stderr, "soundterm OK\n");
}


/*  SOUNDPLAY  --  Begin playing a sound.     
    Most NS Intel drivers are half-duplex only, the problem is that only one DMA channel is used
    for both recording and playing. Some drivers simply go crazy others even bomb out the system
    when driven too hard. Fortunately sf provides a mean to drop (mute) sound frames on a half-
    duplex connection. Whoever designed this SNDDriver BS should be forced to use it until s/he
    is sorry about it - couldn't be very long.
    We simply queue the frames here, no need for SNDWait(), freeing is done by the system.
*/

void soundplay(int len, unsigned char *buf)
{
  sendDataToChild(buf,len);
  return;
}

/*  SOUNDPLAYVOL  --  Set playback volume from 0 (silence) to 100 (full on). */

void soundplayvol(value)
  int value;
{
  int nextvalue;

  nextvalue=value*100/43;

  SNDSetVolume(nextvalue,nextvalue);
}

/*  SOUNDRECGAIN  --  Set  gain from 0 (minimum) to 100 (maximum).  */

void soundrecgain(value)
  int value;
{
  /* set  gain */
}

/*  SOUNDDEST  --  Set destination for generated sound.  If "where"
		   is 0, sound goes to the built-in speaker; if
		   1, to the audio output jack. */

void sounddest(where)
  int where;
{
  /* should choose output device here */
}

/*  SOUNDGRAB  --  Return audio information in the record queue.  */

int soundgrab(unsigned char *buf, int len)
{
    int err,i;

    if (!EternalLife) /* recorder must be down */
    {
	EternalLife=TRUE;
	startIOSubprocess(NULL);
    }
    if (len > 0) {
	i=len;
	//printf("soundgrab--> %d %d\n", GetRPos, InsertRPos);
	while (i--) {
		if (GetRPos==InsertRPos) {
			while (GetRPos==InsertRPos) usleep(100000);
		}
		*buf++ = ringbuffer[GetRPos];
		ringbuffer[GetRPos++]=0; /* clean behind */
		if (GetRPos>=RINGBUFSIZE) GetRPos=0;
	}
    }

    return len;
}

/*  SOUNDFLUSH	--  Flush any queued sound.  */

void soundflush()
{
    //EternalLife=FALSE;
    //kill(childPid, 1);
    Recording=0;
    //fprintf(stderr,"SND FLUSH\n");
}

/* FAKE FUNCTIONS TO MAKE IT HAPPY */

char *cuserid(s)
char *s;
{
	return "fake";
}

#if defined(LIBC_SCCS) && !defined(lint)
/*static char *sccsid = "from: @(#)strdup.c     5.4 (Berkeley) 2/24/91";*/
static char *rcsid = "$Id: soundbyte.c,v 1.1 1997/09/18 13:31:36 dicosmo Exp dicosmo $";
#endif /* LIBC_SCCS and not lint */

#include <stdlib.h>
#include <string.h>

char *
strdup(str)
        const char *str;
{
        size_t len;
        char *copy;

        len = strlen(str) + 1;
        if (!(copy = malloc(len)))
                return((char *)NULL);
        memcpy(copy, str, len);
        return(copy);
}

char *getcwd(buf,size)
char *buf;
size_t size;
{
getwd(buf);
}


#endif
